from collections import deque
from views.mainScreen import MainFrame
from models.data import TextData
from Ai.interence import VeraMindInference
from Ai.llm import ArticleRater

BAD_WORDS = ["FAKE", "SATIRE"]
GOOD_WORDS = ["REAL"]
BAD_COLOR = "#ff8080"
GOOD_COLOR = "#80ff8f"
WORDS = BAD_WORDS + GOOD_WORDS
class MainFrameController:
    
    def __init__(self,frame:MainFrame) -> None:
        self.frame = frame
        self.model_inference = VeraMindInference('VeraMind-Mini')
        self.rater = ArticleRater()
        
    def get_textdata(self) -> TextData:
        text_data = TextData()
        text_data.url = self.frame.entry_url.get()
        if text_data.text_from_url():
            text_data.text = self.frame.input_textbox.get("0.0", "end")
        
        return text_data
    
    def press_check_button(self):
        text_data = self.get_textdata()
        self.prediction(text_data)
        self.frame.output_textbox.configure(state="normal")
        self.frame.output_textbox.delete("0.0", "end")

        response_stream = self.rater.get_response(text_data.text, text_data.confidence, text_data.result)

        highlight_buffer = deque(maxlen=5)

        for chunk in response_stream:
            # Display the chunk immediately
            self.frame.output_textbox.insert("end", chunk)
            self.frame.output_textbox.see("end")
            self.frame.update_idletasks()

            # Add to highlight buffer
            highlight_buffer.append(chunk)

            # Process highlighting when buffer is full
            if len(highlight_buffer) == 5:
                self.process_highlighting(highlight_buffer)

        # Process any remaining chunks in the buffer
        if highlight_buffer:
            self.process_highlighting(highlight_buffer)

        self.frame.output_textbox.configure(state="disabled")

    def process_highlighting(self, highlight_buffer):
        start_index = self.frame.output_textbox.index(f"end-{sum(len(c) for c in highlight_buffer)}c")
        end_index = self.frame.output_textbox.index("end")
        self.highlight_words(start_index, end_index)

        # Keep overlap of 2 chunks
        highlight_buffer = deque(list(highlight_buffer)[-2:], maxlen=5)

    def highlight_words(self, start_index, end_index):
        content = self.frame.output_textbox.get(start_index, end_index)

        for word in WORDS:
            start = 0
            while True:
                pos = content.find(word, start)
                if pos == -1:
                    break
                word_start = f"{start_index}+{pos}c"
                word_end = f"{word_start}+{len(word)}c"
                tag_name = f"{word.lower()}_color"
                self.frame.output_textbox.tag_add(tag_name, word_start, word_end)
                if word in BAD_WORDS:
                    self.frame.output_textbox.tag_config(tag_name, foreground=BAD_COLOR)
                elif word in GOOD_COLOR:
                    self.frame.output_textbox.tag_config(tag_name, foreground=GOOD_COLOR)
                start = pos + len(word)

        
    def prediction(self, text_data:TextData) -> TextData:
        result = self.model_inference.predict(text_data.text)
        text_data.confidence = result["confidence"]
        text_data.result = result["result"] 
        print(f"Prediction: {text_data.result}")
        print(f"Confidence: {text_data.confidence}")
        return text_data